#!/bin/bash
set -ex

export OS_CLOUD=openstack_helm
# Create a security group.
security_group_state=`openstack security group list | grep security1 | awk '{print $4}'`
if [[ $security_group_state != "security1" ]]
then
    openstack security group create security1
    openstack security group rule create --ingress --protocol icmp --remote-ip 0.0.0.0/0 security1
    openstack security group rule create --ingress --protocol tcp --remote-ip 0.0.0.0/0 security1
    openstack security group rule create --ingress --protocol udp --remote-ip 0.0.0.0/0 security1
fi

# When the instance migration fails, stop this loop statement
test_status="true"
migration_count=0
while $test_status
do
# Create two instances and check if they are active.
vm_name_0=`openstack server list | grep vm0 | awk '{print $4}'`
vm_name_1=`openstack server list | grep vm1 | awk '{print $4}'`
if [[ $vm_name_0 = "" ]] && [[ $vm_name_1 = "" ]]
then
    openstack server create --image cirros --flavor m1.tiny --network public-net0 vm0 --security-group security1
    openstack server create --image cirros --flavor m1.tiny --network public-net0 vm1 --security-group security1
    for i in vm0 vm1
    do 
        vm_state=""
        while [[ $vm_state != "ACTIVE" ]]
        do
            vm_state=`openstack server list | grep $i | awk '{print $6}'`
            sleep 2
        done
    done
else
    echo "vm0 and vm1 is already exists!"
    exit
fi
openstack server list

# Get ip of vm0 and vm1.
s0=`openstack server list | grep vm0 | awk '{print $8}'`
s1=`openstack server list | grep vm1 | awk '{print $8}'`
vm0_ip=${s0:12}
vm1_ip=${s1:12}

# Login to the compute-0 node and ping the vm0 and vm1‘ip from the router namespace, ensure that the network is normal before migration.
for ips in $vm0_ip $vm1_ip
do
tee ./test.sh << EOF
sudo ip netns exec qrouter-a33b79cd-41bc-47a0-bd82-e4f00688c414 bash
ip=$ips
    for num in \$(seq 1 100)
    do
        result=\`ping -w 4 -c 4 \$ip | grep packet | awk -F" " '{print \$6}'| awk -F"%" '{print \$1}'| awk -F' ' '{print \$1}'\`
        if [[ \$result -eq 0 ]] && [[ \$num -le 50 ]]
        then
            echo "succeeded"
            exit
        elif [[ \$result -ne 0 ]] && [[ \$num -le 50 ]]
        then
            num=\$num+1
            sleep 3
        else
            echo "failed"
            exit
        fi
    done
EOF
ssh -t compute-1 < test.sh >> ./vm_state.txt
done
if [[ `cat vm_state.txt | awk '{print $1}' | sed -n '1p'` = "failed" ]] || [[ `cat vm_state.txt | awk '{print $1}' | sed -n '2p'` = "failed" ]]
then
    echo "Ping failed before live migration"
    exit
fi
echo "Ping succeeded before live migration"
rm -rf ./vm_state.txt
rm -rf ./test.sh

# Live migration of vm1 instance.
openstack server show vm1
vm_host=`openstack server show vm1 | grep OS-EXT-SRV-ATTR:host | awk '{print $4}'`
if [ $vm_host = "compute-0" ]
then
    openstack server migrate vm1 --live compute-1 --block-migration
elif [ $vm_host = "compute-1" ]
then
    openstack server migrate vm1 --live compute-2 --block-migration
else
    openstack server migrate vm1 --live compute-0 --block-migration
fi

# Check if vm1 is migrated.
migration_state="migrating"
while [[ $migration_state = "migrating" ]]
do
    migration_state=`openstack server show vm1 | grep OS-EXT-STS:task_state | awk '{print $4}'`
    openstack server show vm1
    sleep 2
done
echo "vm1 live migration succeeded"
migration_count=$[$migration_count+1]
echo "vm1 has been migrated "$migration_count" times"

# Get ip of vm1.
s1=`openstack server list | grep vm1 | awk '{print $8}'`
vm1_ip=${s1:12}

# Login to the compute-0 node and ping the vm1‘ip from the router namespace, confirm the network is normal after migration.
tee ./test.sh << EOF
sudo ip netns exec qrouter-a33b79cd-41bc-47a0-bd82-e4f00688c414 bash
ip=$vm1_ip
for num in \$(seq 1 100)
do
    result=\`ping -w 4 -c 4 \$ip | grep packet | awk -F" " '{print \$6}'| awk -F"%" '{print \$1}'| awk -F' ' '{print \$1}'\`
    if [[ \$result -eq 0 ]] && [[ \$num -le 50 ]]
    then
        echo "succeeded"
        exit
    elif [[ \$result -ne 0 ]] && [[ \$num -le 50 ]]
    then
        num=\$num+1
        sleep 3
    else
        echo "failed"
        exit
    fi
done
EOF
ssh -t compute-1 < test.sh >> ./vm_state.txt

# After confirming that the network is normal, delete the two instances.
if [ `cat vm_state.txt` = "succeeded" ]
then
    openstack server delete vm0
    openstack server delete vm1
    vm_name_0="test0"
    vm_name_1="test1"
    while [[ $vm_name_0 != "" ]] || [[ $vm_name_1 != "" ]]
    do
        vm_name_0=`openstack server list | grep vm0 | awk '{print $4}'`
        vm_name_1=`openstack server list | grep vm1 | awk '{print $4}'`
        sleep 2
    done
    echo "Ping succeeded after vm1 live migration"
    rm -rf ./vm_state.txt
    rm -rf ./test.sh
    test_status="true"
else
    echo "Ping failed after vm1 live migration"
    test_status="false"
fi
done
